@@ -23,9 +23,9 @@ class.  For those wishing to override the braille generators, however,
 one can create a new instance and replace/extend the braille
 generators as they see fit."""
 
-__id__        = "$Id: braillegenerator.py,v 1.62 2006/10/28 16:23:33 richb Exp $"
-__version__   = "$Revision: 1.62 $"
-__date__      = "$Date: 2006/10/28 16:23:33 $"
+__id__        = "$Id: braillegenerator.py,v 1.63 2006/11/16 19:11:57 wwalker Exp $"
+__version__   = "$Revision: 1.63 $"
+__date__      = "$Date: 2006/11/16 19:11:57 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -223,7 +223,11 @@ class BrailleGenerator:
         try:
             minimumIncrement = value.minimumIncrement
         except:
-            minimumIncrement = (value.maximumValue - value.minimumValue) / 100.0
+            pass
+
+        if minimumIncrement == 0.0:
+            minimumIncrement = (value.maximumValue - value.minimumValue) \
+                               / 100.0
 
         try:
             decimalPlaces = max(0, -math.log10(minimumIncrement))
