@@ -19,9 +19,9 @@
 
 """Provides the FocusTrackingPresenter for Orca."""
 
-__id__        = "$Id: focus_tracking_presenter.py,v 1.90 2006/11/01 13:09:34 wwalker Exp $"
-__version__   = "$Revision: 1.90 $"
-__date__      = "$Date: 2006/11/01 13:09:34 $"
+__id__        = "$Id: focus_tracking_presenter.py,v 1.93 2006/11/13 22:33:00 richb Exp $"
+__version__   = "$Revision: 1.93 $"
+__date__      = "$Date: 2006/11/13 22:33:00 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -68,6 +68,8 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         #
         self.registry        = atspi.Registry()
         self._knownScripts   = {}
+        self._knownAppSettings = {}
+        self._oldAppSettings = None
         self._eventQueue     = Queue.Queue(0)
         self._gidleId        = 0
         self._gidleLock      = threading.Lock()
@@ -294,6 +296,44 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         except:
             debug.printException(debug.LEVEL_FINEST)
 
+    ########################################################################
+    #                                                                      #
+    # METHODS FOR KEEPING TRACK OF APPLICATION SETTINGS.                   #
+    #                                                                      #
+    ########################################################################
+
+    def loadAppSettings(self, app):
+        """Load the users application specific settings for an app.
+
+        Arguments:
+        - app: the Python app
+        """
+
+        settingsPackages = settings.settingsPackages
+        moduleName = settings.getScriptModuleName(app)
+        module = None
+
+        if moduleName and len(moduleName):
+            for package in settingsPackages:
+                if len(package):
+                    name = package + "." + moduleName
+                else:
+                    name = moduleName
+                try:
+                    debug.println(debug.LEVEL_FINEST,
+                                  "Looking for settings at %s.py..." % name)
+                    if self. _knownAppSettings.has_key(name):
+                        reload(self._knownAppSettings[name])
+                    else:
+                        self._knownAppSettings[name] = \
+                            __import__(name, globals(), locals(), [''])
+                    debug.println(debug.LEVEL_FINEST,
+                                  "...found %s.py" % name)
+                    break
+                except ImportError:
+                    debug.println(debug.LEVEL_FINEST,
+                                  "...could not find %s.py" % name)
+
     ########################################################################
     #                                                                      #
     # METHODS FOR PRE-PROCESSING AND MASSAGING AT-SPI OBJECT EVENTS        #
@@ -429,6 +469,15 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                     if (event.type == "window:activate") \
                        or ((event.type == "focus:") \
                            and (event.source.role == rolenames.ROLE_FRAME)):
+
+                        # If old ("factory") settings don't exist yet, save 
+                        # a set, else restore the old application settings.
+                        #
+                        if not self._oldAppSettings:
+                            self._oldAppSettings = util.saveOldAppSettings()
+                        else:
+                            util.restoreOldAppSettings(self._oldAppSettings)
+
                         # We'll let someone else decide if it's important
                         # to stop speech or not.
                         #speech.stop()
@@ -436,6 +485,12 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                             self._getScript(event.source.app)
                         debug.println(debug.LEVEL_FINE, "ACTIVE SCRIPT: " \
                                       + orca_state.activeScript.name)
+
+                        # Load in the application specific settings for the
+                        # app for this event (if found).
+                        #
+                        appSettings = self.loadAppSettings(event.source.app)
+
                     s = self._getScript(event.source.app)
                     s.processObjectEvent(event)
                     if retryCount:
@@ -766,6 +821,8 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
 
         self._listenerCounts = {}
         self._knownScripts   = {}
+        self._knownAppSettings = {}
+        self._oldAppSettings = None
         self._defaultScript  = None
 
         orca_state.activeScript = self._getScript(None)
@@ -795,6 +852,8 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                                                   eventType)
         self._listenerCounts = {}
         self._knownScripts   = {}
+        self._knownAppSettings = {}
+        self._oldAppSettings = None
         self._defaultScript  = None
 
         orca_state.activeScript = None
