@@ -20,9 +20,9 @@
 """Holds state that is shared among many modules.
 """
 
-__id__        = "$Id: orca_state.py,v 1.5 2006/10/27 16:25:44 mpeders Exp $"
-__version__   = "$Revision: 1.5 $"
-__date__      = "$Date: 2006/10/27 16:25:44 $"
+__id__        = "$Id: orca_state.py,v 1.6 2006/11/30 16:38:58 richb Exp $"
+__version__   = "$Revision: 1.6 $"
+__date__      = "$Date: 2006/11/30 16:38:58 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -68,3 +68,8 @@ noFocusTimestamp = 0.0
 # The last word spoken.
 #
 lastWord = ""
+
+# The last searchQuery
+#
+searchQuery = None
+
