@@ -21,9 +21,9 @@
 """Provides a methods that converts the role name of an Accessible
 object into localized strings for speech and braille."""
 
-__id__        = "$Id: rolenames.py,v 1.29 2006/11/01 22:55:51 wwalker Exp $"
-__version__   = "$Revision: 1.29 $"
-__date__      = "$Date: 2006/11/01 22:55:51 $"
+__id__        = "$Id: rolenames.py,v 1.30 2006/11/10 22:46:57 wwalker Exp $"
+__version__   = "$Revision: 1.30 $"
+__date__      = "$Date: 2006/11/10 22:46:57 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -44,6 +44,7 @@ ROLE_ALERT               = "alert"
 ROLE_ANIMATION           = "animation"
 ROLE_ARROW               = "arrow"
 ROLE_CALENDAR            = "calendar"
+ROLE_CAPTION             = "caption"
 ROLE_CANVAS              = "canvas"
 ROLE_CHECK_BOX           = "check box"
 ROLE_CHECK_MENU_ITEM     = "check menu item"
@@ -185,6 +186,11 @@ rolenames[ROLE_CANVAS] = Rolename(ROLE_CANVAS,
                                   _("Canvas"),
                                   _("canvas"))
 
+rolenames[ROLE_CAPTION] = Rolename(ROLE_CAPTION,
+                                   _("cptn"),
+                                   _("Caption"),
+                                   _("caption"))
+
 rolenames[ROLE_CHECK_BOX] = Rolename(ROLE_CHECK_BOX,
                                      _("chk"),
                                      _("CheckBox"),
