@@ -19,9 +19,9 @@
 
 """Provides various utility functions for Orca."""
 
-__id__        = "$Id: util.py,v 1.38 2006/08/23 12:30:59 oserb Exp $"
-__version__   = "$Revision: 1.38 $"
-__date__      = "$Date: 2006/08/23 12:30:59 $"
+__id__        = "$Id: util.py,v 1.53 2006/11/30 03:02:23 lmonsanto Exp $"
+__version__   = "$Revision: 1.53 $"
+__date__      = "$Date: 2006/11/30 03:02:23 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -38,9 +38,14 @@ except:
 import string
 
 import atspi
+import chnames
 import debug
 import input_event
+import orca_prefs
 import orca_state
+import phonnames
+import punctuation_settings
+import pronunciation_dict
 import rolenames
 import settings
 import speech
@@ -48,6 +53,11 @@ import speechserver
 
 from orca_i18n import _ # for gettext support
 
+# Embedded object character used to indicate that an object is
+# embedded in a string.
+#
+EMBEDDED_OBJECT_CHARACTER = u'\ufffc'
+
 def isSameObject(obj1, obj2):
     if (obj1 == obj2):
         return True
@@ -55,27 +65,27 @@ def isSameObject(obj1, obj2):
         return False
 
     try:
-	if obj1.name != obj2.name:
-	    return False
-	
-	# When we're looking at children of objects that manage 
-	# their descendants, we will often get different objects 
-	# that point to the same logical child.  We want to be able 
-	# to determine if two objects are in fact pointing to the same child.
-	# If we cannot do so easily (i.e., object equivalence), we examine 
-	# the hierarchy and the object index at each level.
-	#
-	parent1 = obj1
-	parent2 = obj2
-	while (parent1 and parent2 and \
-		parent1.state.count(atspi.Accessibility.STATE_TRANSIENT) and \
-		parent2.state.count(atspi.Accessibility.STATE_TRANSIENT)):
-	    if parent1.index != parent2.index:
-		return False
-	    parent1 = parent1.parent
-	    parent2 = parent2.parent
-	if parent1 and parent2 and parent1 == parent2:
-	    return True
+        if obj1.name != obj2.name:
+            return False
+
+        # When we're looking at children of objects that manage
+        # their descendants, we will often get different objects
+        # that point to the same logical child.  We want to be able
+        # to determine if two objects are in fact pointing to the same child.
+        # If we cannot do so easily (i.e., object equivalence), we examine
+        # the hierarchy and the object index at each level.
+        #
+        parent1 = obj1
+        parent2 = obj2
+        while (parent1 and parent2 and \
+                parent1.state.count(atspi.Accessibility.STATE_TRANSIENT) and \
+                parent2.state.count(atspi.Accessibility.STATE_TRANSIENT)):
+            if parent1.index != parent2.index:
+                return False
+            parent1 = parent1.parent
+            parent2 = parent2.parent
+        if parent1 and parent2 and parent1 == parent2:
+            return True
     except:
         pass
 
@@ -83,18 +93,18 @@ def isSameObject(obj1, obj2):
     # (fix for bug #352250)
     #
     try:
-	parent1 = obj1
-	parent2 = obj2
-	while parent1 and parent2 and \
-		parent1.role == rolenames.ROLE_LABEL and \
-		parent2.role == rolenames.ROLE_LABEL:
-	    parent1 = parent1.parent
-	    parent2 = parent2.parent
-	if parent1 and parent2 and parent1 == parent2:
-	    return True
+        parent1 = obj1
+        parent2 = obj2
+        while parent1 and parent2 and \
+                parent1.role == rolenames.ROLE_LABEL and \
+                parent2.role == rolenames.ROLE_LABEL:
+            parent1 = parent1.parent
+            parent2 = parent2.parent
+        if parent1 and parent2 and parent1 == parent2:
+            return True
     except:
-	pass
-    
+        pass
+
     return False
 
 def appendString(text, newText, delimiter=" "):
@@ -280,7 +290,7 @@ def __getDisplayedTextInComboBox(combo):
             textObj = child
 
     if textObj:
-        [displayedText, startOffset, endOffset] = getTextLineAtCaret(textObj)
+        [displayedText, caretOffset, startOffset] = getTextLineAtCaret(textObj)
         #print "TEXTOBJ", displayedText
     else:
         selectedItem = None
@@ -303,7 +313,8 @@ def __getDisplayedTextInComboBox(combo):
             displayedText = combo.name
             #print "NAME", displayedText
         elif combo.text:
-            [displayedText, startOffset, endOffset] = getTextLineAtCaret(combo)
+            [displayedText, caretOffset, startOffset] = \
+                getTextLineAtCaret(combo)
             #print "TEXT", displayedText
 
     return displayedText
@@ -406,9 +417,9 @@ def findFocusedObject(root):
 
     return None
 
-def isDoubleClick(lastInputEvent, inputEvent):
-    """Return an indication of whether the user has double-clicked a
-    one of the keys on the keyboard.
+def getClickCount(lastInputEvent, inputEvent):
+    """Return the count of the number of clicks a user has made to one of the
+    keys on the keyboard.
 
     Arguments:
     - lastInputEvent: the last input event.
@@ -417,16 +428,19 @@ def isDoubleClick(lastInputEvent, inputEvent):
 
     if not isinstance(lastInputEvent, input_event.KeyboardEvent) or \
        not isinstance(inputEvent, input_event.KeyboardEvent):
-        return False
+        orca_state.clickCount = 0
+        return orca_state.clickCount
 
     if (lastInputEvent.hw_code != inputEvent.hw_code) or \
        (lastInputEvent.modifiers != inputEvent.modifiers):
-        return False
+        orca_state.clickCount = 1
+        return orca_state.clickCount
 
     if (inputEvent.time - lastInputEvent.time) < settings.doubleClickTimeout:
-        return True
+        orca_state.clickCount += 1
     else:
-        return False
+        orca_state.clickCount = 0
+    return orca_state.clickCount+1
 
 def isDesiredFocusedItem(obj, rolesList):
     """Called to determine if the given object and it's hierarchy of
@@ -533,14 +547,15 @@ def textLines(obj):
             lastEndOffset = endOffset
             offset = endOffset
 
-            # Strip trailing new lines
-            #
-            #if string[-1:] == "\n":
-            #    string = string[0][:-1]
+            string = adjustForRepeats(string)
+            if string.isupper():
+                voice = settings.voices[settings.UPPERCASE_VOICE]
+            else:
+                voice = settings.voices[settings.DEFAULT_VOICE]
 
             yield [speechserver.SayAllContext(obj, string,
                                               startOffset, endOffset),
-                   None]
+                   voice]
 
         moreLines = False
         relations = obj.relations
@@ -560,6 +575,108 @@ def textLines(obj):
         if not moreLines:
             done = True
 
+def _addRepeatSegment(segment, line, respectPunctuation=True):
+    """Add in the latest line segment, adjusting for repeat characters
+    and punctuation.
+
+    Arguments:
+    - segment: the segment of repeated characters.
+    - line: the current built-up line to characters to speak.
+    - respectPunctuation: if False, ignore punctuation level.
+
+    Returns: the current built-up line plus the new segment, after
+    adjusting for repeat character counts and punctuation.
+    """
+
+    style = settings.verbalizePunctuationStyle
+    isPunctChar = True
+    try:
+        level, action = punctuation_settings.getPunctuationInfo(segment[0])
+    except:
+        isPunctChar = False
+    count = len(segment)
+    if (count >= settings.repeatCharacterLimit) \
+       and (not segment[0] in string.whitespace):
+        if (not respectPunctuation) \
+           or (isPunctChar and (style <= level)):
+            repeatChar = chnames.getCharacterName(segment[0])
+            line += _(" %d %s characters ") % (count, repeatChar)
+    else:
+        line += segment
+
+    return line
+
+def adjustForRepeats(line):
+    """Adjust line to include repeat character counts.
+    As some people will want this and others might not,
+    there is a setting in settings.py that determines
+    whether this functionality is enabled.
+
+    repeatCharacterLimit = <n>
+
+    If <n> is 0, then there would be no repeat characters.
+    Otherwise <n> would be the number of same characters (or more)
+    in a row that cause the repeat character count output.
+    If the value is set to 1, 2 or 3 then it's treated as if it was
+    zero. In other words, no repeat character count is given.
+
+    Arguments:
+    - line: the string to adjust for repeat character counts.
+
+    Returns: a new line adjusted for repeat character counts (if enabled).
+    """
+
+    line = line.decode("UTF-8")
+
+    if (len(line) < 4) or (settings.repeatCharacterLimit < 4):
+        return line.encode("UTF-8")
+
+    newLine = u''
+    segment = lastChar = line[0]
+
+    multipleChars = False
+    for i in range(1, len(line)):
+        if line[i] == lastChar:
+            segment += line[i]
+        else:
+            multipleChars = True
+            newLine = _addRepeatSegment(segment, newLine)
+            segment = line[i]
+
+        lastChar = line[i]
+
+    newLine = _addRepeatSegment(segment, newLine, multipleChars)
+
+    return newLine.encode("UTF-8")
+
+def adjustForPronunciation(line):
+    """Adjust the line to replace words in the pronunciation dictionary,
+    with what those words actually sound like.
+
+    Arguments:
+    - line: the string to adjust for words in the pronunciation dictionary.
+
+    Returns: a new line adjusted for words found in the pronunciation
+    dictionary.
+    """
+
+    line = line.decode("UTF-8")
+    newLine = segment = u''
+
+    for i in range(0, len(line)):
+        if isWordDelimiter(line[i]):
+            if len(segment) != 0:
+                newLine = newLine + pronunciation_dict.getPronunciation(segment)
+            newLine = newLine + line[i]
+            segment = u''
+        else:
+            segment += line[i]
+
+    if len(segment) != 0:
+        newLine = newLine + pronunciation_dict.getPronunciation(segment)
+
+    return newLine.encode("UTF-8")
+
 def getLinkIndex(obj, characterIndex):
     """A brute force method to see if an offset is a link.  This
     is provided because not all Accessible Hypertext implementations
@@ -599,8 +716,11 @@ def isWordDelimiter(character):
     Returns True if the given character is a word delimiter.
     """
 
+    if isinstance(character, unicode):
+        character = character.encode("UTF-8")
+
     return (character in string.whitespace) \
-           or (character in string.punctuation)
+           or (character in '!*+,-./:;<=>?@[\]^_{|}')
 
 def getFrame(obj):
     """Returns the frame containing this object, or None if this object
@@ -628,6 +748,33 @@ def getFrame(obj):
 
     return obj
 
+def getTopLevel(obj):
+    """Returns the top-level object (frame, dialog ...) containing this
+    object, or None if this object is not inside a top-level object.
+
+    Arguments:
+    - obj: the Accessible object
+    """
+
+    debug.println(debug.LEVEL_FINEST,
+                  "Finding top-level object for source.name="
+                  + obj.accessibleNameToString())
+
+    while obj \
+          and obj.parent \
+          and (obj != obj.parent) \
+          and (obj.parent.role != rolenames.ROLE_APPLICATION):
+        obj = obj.parent
+        debug.println(debug.LEVEL_FINEST, "--> obj.name="
+                      + obj.accessibleNameToString())
+
+    if obj and obj.parent and (obj.parent.role == rolenames.ROLE_APPLICATION):
+        pass
+    else:
+        obj = None
+
+    return obj
+
 def getTextLineAtCaret(obj):
     """Gets the line of text where the caret is.
 
@@ -635,7 +782,8 @@ def getTextLineAtCaret(obj):
     - obj: an Accessible object that implements the AccessibleText
            interface
 
-    Returns the line of text where the caret is.
+    Returns the [string, caretOffset, startOffset] for the line of text
+    where the caret is.
     """
 
     # Get the the AccessibleText interrface
@@ -646,20 +794,16 @@ def getTextLineAtCaret(obj):
 
     # Get the line containing the caret
     #
-    offset = text.caretOffset
-    line = text.getTextAtOffset(offset,
-                                atspi.Accessibility.TEXT_BOUNDARY_LINE_START)
+    [string, startOffset, endOffset] = text.getTextAtOffset(
+        text.caretOffset, atspi.Accessibility.TEXT_BOUNDARY_LINE_START)
 
-    # Line is actually a list of objects-- the first is the actual
-    # text of the line, the second is the start offset, and the third
-    # is the end offset.  Sometimes we get the trailing line-feed-- remove it
+    # Sometimes we get the trailing line-feed-- remove it
     #
-    if line[0][-1:] == "\n":
-        content = line[0][:-1]
-    else:
-        content = line[0]
+    content = string.decode("UTF-8")
+    if content[-1:] == "\n":
+        content = content[:-1]
 
-    return [content, offset, line[1]]
+    return [content.encode("UTF-8"), text.caretOffset, startOffset]
 
 def getNodeLevel(obj):
     """Determines the node level of this object if it is in a tree
@@ -897,6 +1041,22 @@ def findUnrelatedLabels(root):
 
     return sortedLabels
 
+def phoneticSpellCurrentItem(string):
+    """Phonetically spell the current flat review word or line.
+
+    Arguments:
+    - string: the string to phonetically spell.
+    """
+
+    for (index, character) in enumerate(string.decode("UTF-8")):
+        if character.isupper():
+            voice = settings.voices[settings.UPPERCASE_VOICE]
+            character = character.lower()
+        else:
+            voice =  settings.voices[settings.DEFAULT_VOICE]
+        string = phonnames.getPhoneticName(character)
+        speech.speak(string, voice)
+
 def printAncestry(child):
    """Prints a hierarchical view of a child's ancestry."""
 
@@ -1029,6 +1189,23 @@ def findActiveWindow():
 
     return window
 
+def saveOldAppSettings():
+    """Save a copy of all the existing application specific settings
+    (as specified by the settings.userCustomizableSettings dictionary)."""
+
+    return orca_prefs.readPreferences()
+
+def restoreOldAppSettings(prefsDict):
+    """Restore a copy of all the previous saved application settings.
+
+    Arguments:
+    - prefsDict: the dictionary containing the old application settings.
+    """
+
+    for key in settings.userCustomizableSettings:
+        if prefsDict.has_key(key):
+            settings.__dict__[key] = prefsDict[key]
+
 ########################################################################
 #                                                                      #
 # METHODS FOR DRAWING RECTANGLES AROUND OBJECTS ON THE SCREEN          #
@@ -1200,23 +1377,24 @@ def speakTextSelectionState(obj, startOffset, endOffset):
         # characters on either end of the text. We adjust the startOffset and
         # endOffset to exclude them.
         #
-        str = obj.text.getText(startOffset, endOffset)
+        try:
+            str = obj.text.getText(startOffset, endOffset).decode("UTF-8")
+        except:
+            str = u''
         n = len(str)
 
         # Don't strip whitespace if string length is one (might be a space).
         #
         if n > 1:
             while endOffset > startOffset:
-                if str[n-1] in string.whitespace or \
-                   str[n-1] in string.punctuation:
+                if isWordDelimiter(str[n-1]):
                     n -= 1
                     endOffset -= 1
                 else:
                     break
             n = 0
             while startOffset < endOffset:
-                if str[n] in string.whitespace or \
-                   str[n] in string.punctuation:
+                if isWordDelimiter(str[n]):
                     n += 1
                     startOffset += 1
                 else:
